<?php

namespace Application;

use Application\Controller\ApplicationController;
use Application\Controller\ApplicationControllerFactory;
use Application\Form\Application\ApplicationForm;
use Application\Form\Application\ApplicationFormFactory;
use Application\Form\Application\ApplicationHydrator;
use Application\Form\Application\ApplicationHydratorFactory;
use Application\Form\AssocierApplications\AssocierApplicationFormFactory;
use Application\Form\AssocierApplications\AssocierApplicationsForm;
use Application\Form\AssocierApplications\AssocierApplicationsHydrator;
use Application\Form\AssocierApplications\AssocierApplicationsHydratorFactory;
use Application\Provider\Privileges\ApplicationPrivileges;
use Application\Service\Application\ApplicationService;
use Application\Service\Application\ApplicationServiceFactory;
use Laminas\Router\Http\Literal;
use Laminas\Router\Http\Segment;
use UnicaenPrivilege\Guard\PrivilegeController;

return [
    'bjyauthorize' => [
        'guards' => [
            PrivilegeController::class => [
                [
                    'controller' => ApplicationController::class,
                    'action' => [
                        'index',
                    ],
                    'privileges' => [
                        ApplicationPrivileges::APPLICATION_INDEX,
                    ],
                ],
                [
                    'controller' => ApplicationController::class,
                    'action' => [
                        'afficher',
                    ],
                    'privileges' => [
                        ApplicationPrivileges::APPLICATION_AFFICHER,
                    ],
                ],
                [
                    'controller' => ApplicationController::class,
                    'action' => [
                        'ajouter',
                    ],
                    'privileges' => [
                        ApplicationPrivileges::APPLICATION_AJOUTER,
                    ],
                ],
                [
                    'controller' => ApplicationController::class,
                    'action' => [
                        'modifier',
                        'associer-categories',
                    ],
                    'privileges' => [
                        ApplicationPrivileges::APPLICATION_MODIFIER,
                    ],
                ],
                [
                    'controller' => ApplicationController::class,
                    'action' => [
                        'historiser',
                        'restaurer',
                    ],
                    'privileges' => [
                        ApplicationPrivileges::APPLICATION_HISTORISER,
                    ],
                ],
                [
                    'controller' => ApplicationController::class,
                    'action' => [
                        'supprimer',
                    ],
                    'privileges' => [
                        ApplicationPrivileges::APPLICATION_SUPPRIMER,
                    ],
                ],
            ],
        ],
    ],

    'router'          => [
        'routes' => [
            'application' => [
                'type'  => Literal::class,
                'options' => [
                    'route'    => '/application',
                    'defaults' => [
                        /** @see ApplicationController::indexAction() */
                        'controller' => ApplicationController::class,
                        'action'     => 'index',
                    ],
                ],
                'may_terminate' => true,
                'child_routes' => [
                    'afficher' => [
                        'type'  => Segment::class,
                        'options' => [
                            'route'    => '/afficher/:application',
                            'defaults' => [
                                /** @see ApplicationController::afficherAction() */
                                'action'     => 'afficher',
                            ],
                        ],
                    ],
                    'ajouter' => [
                        'type'  => Literal::class,
                        'options' => [
                            'route'    => '/ajouter',
                            'defaults' => [
                                /** @see ApplicationController::ajouterAction() */
                                'action'     => 'ajouter',
                            ],
                        ],
                    ],
                    'modifier' => [
                        'type'  => Segment::class,
                        'options' => [
                            'route'    => '/modifier/:application',
                            'defaults' => [
                                /** @see ApplicationController::modifierAction() */
                                'action'     => 'modifier',
                            ],
                        ],
                    ],
                    'associer-categories' => [
                        'type'  => Segment::class,
                        'options' => [
                            'route'    => '/associer-categories/:application',
                            'defaults' => [
                                /** @see ApplicationController::associerCategoriesAction() */
                                'action'     => 'associer-categories',
                            ],
                        ],
                    ],
                    'historiser' => [
                        'type'  => Segment::class,
                        'options' => [
                            'route'    => '/historiser/:application',
                            'defaults' => [
                                /** @see ApplicationController::historiserAction() */
                                'action'     => 'historiser',
                            ],
                        ],
                    ],
                    'restaurer' => [
                        'type'  => Segment::class,
                        'options' => [
                            'route'    => '/restaurer/:application',
                            'defaults' => [
                                /** @see ApplicationController::restaurerAction() */
                                'action'     => 'restaurer',
                            ],
                        ],
                    ],
                    'supprimer' => [
                        'type'  => Segment::class,
                        'options' => [
                            'route'    => '/supprimer/:application',
                            'defaults' => [
                                /** @see ApplicationController::supprimerAction() */
                                'action'     => 'supprimer',
                            ],
                        ],
                    ],
                ],
            ],
        ],
    ],

    'service_manager' => [
        'factories' => [
            ApplicationService::class => ApplicationServiceFactory::class,
        ],
    ],
    'controllers'     => [
        'factories' => [
            ApplicationController::class => ApplicationControllerFactory::class,
        ],
    ],
    'form_elements' => [
        'factories' => [
            ApplicationForm::class => ApplicationFormFactory::class,
            AssocierApplicationsForm::class => AssocierApplicationFormFactory::class,
        ],
    ],
    'hydrators' => [
        'factories' => [
            ApplicationHydrator::class => ApplicationHydratorFactory::class,
            AssocierApplicationsHydrator::class => AssocierApplicationsHydratorFactory::class,
        ],
    ],

];