<?php

namespace Application;

use Application\Controller\CategorieController;
use Application\Controller\CategorieControllerFactory;
use Application\Form\AssocierCategories\AssocierCategoriesForm;
use Application\Form\AssocierCategories\AssocierCategoriesFormFactory;
use Application\Form\AssocierCategories\AssocierCategoriesHydrator;
use Application\Form\AssocierCategories\AssocierCategoriesHydratorFactory;
use Application\Form\Categorie\CategorieForm;
use Application\Form\Categorie\CategorieFormFactory;
use Application\Form\Categorie\CategorieHydrator;
use Application\Form\Categorie\CategorieHydratorFactory;
use Application\Provider\Privileges\CategoriePrivileges;
use Application\Service\Categorie\CategorieService;
use Application\Service\Categorie\CategorieServiceFactory;
use Laminas\Router\Http\Literal;
use Laminas\Router\Http\Segment;
use UnicaenPrivilege\Guard\PrivilegeController;

return [
    'bjyauthorize' => [
        'guards' => [
            PrivilegeController::class => [
                [
                    'controller' => CategorieController::class,
                    'action' => [
                        'index',
                    ],
                    'privileges' => [
                        CategoriePrivileges::CATEGORIE_INDEX,
                    ],
                ],
                [
                    'controller' => CategorieController::class,
                    'action' => [
                        'afficher',
                    ],
                    'privileges' => [
                        CategoriePrivileges::CATEGORIE_AFFICHER,
                    ],
                ],
                [
                    'controller' => CategorieController::class,
                    'action' => [
                        'ajouter',
                    ],
                    'privileges' => [
                        CategoriePrivileges::CATEGORIE_AJOUTER,
                    ],
                ],
                [
                    'controller' => CategorieController::class,
                    'action' => [
                        'modifier',
                        'associer-applications',
                    ],
                    'privileges' => [
                        CategoriePrivileges::CATEGORIE_MODIFIER,
                    ],
                ],
                [
                    'controller' => CategorieController::class,
                    'action' => [
                        'historiser',
                        'restaurer',
                    ],
                    'privileges' => [
                        CategoriePrivileges::CATEGORIE_HISTORISER,
                    ],
                ],
                [
                    'controller' => CategorieController::class,
                    'action' => [
                        'supprimer',
                    ],
                    'privileges' => [
                        CategoriePrivileges::CATEGORIE_SUPPRIMER,
                    ],
                ],
            ],
        ],
    ],

    'router' => [
        'routes' => [
            'categorie' => [
                'type' => Literal::class,
                'options' => [
                    'route' => '/categorie',
                    'defaults' => [
                        /** @see CategorieController::indexAction() */
                        'controller' => CategorieController::class,
                        'action' => 'index',
                    ],
                ],
                'may_terminate' => true,
                'child_routes' => [
                    'afficher' => [
                        'type' => Segment::class,
                        'options' => [
                            'route' => '/afficher/:categorie',
                            'defaults' => [
                                /** @see CategorieController::afficherAction() */
                                'action' => 'afficher',
                            ],
                        ],
                    ],
                    'ajouter' => [
                        'type' => Literal::class,
                        'options' => [
                            'route' => '/ajouter',
                            'defaults' => [
                                /** @see CategorieController::ajouterAction() */
                                'action' => 'ajouter',
                            ],
                        ],
                    ],
                    'modifier' => [
                        'type' => Segment::class,
                        'options' => [
                            'route' => '/modifier/:categorie',
                            'defaults' => [
                                /** @see CategorieController::modifierAction() */
                                'action' => 'modifier',
                            ],
                        ],
                    ],
                    'associer-applications' => [
                        'type' => Segment::class,
                        'options' => [
                            'route' => '/associer-applications/:categorie',
                            'defaults' => [
                                /** @see CategorieController::associerApplicationsAction() */
                                'action' => 'associer-applications',
                            ],
                        ],
                    ],
                    'historiser' => [
                        'type' => Segment::class,
                        'options' => [
                            'route' => '/historiser/:categorie',
                            'defaults' => [
                                /** @see CategorieController::historiserAction() */
                                'action' => 'historiser',
                            ],
                        ],
                    ],
                    'restaurer' => [
                        'type' => Segment::class,
                        'options' => [
                            'route' => '/restaurer/:categorie',
                            'defaults' => [
                                /** @see CategorieController::restaurerAction() */
                                'action' => 'restaurer',
                            ],
                        ],
                    ],
                    'supprimer' => [
                        'type' => Segment::class,
                        'options' => [
                            'route' => '/supprimer/:categorie',
                            'defaults' => [
                                /** @see CategorieController::supprimerAction() */
                                'action' => 'supprimer',
                            ],
                        ],
                    ],
                ],
            ],
        ],
    ],

    'service_manager' => [
        'factories' => [
            CategorieService::class => CategorieServiceFactory::class,
        ],
    ],
    'controllers' => [
        'factories' => [
            CategorieController::class => CategorieControllerFactory::class,
        ],
    ],
    'form_elements' => [
        'factories' => [
            AssocierCategoriesForm::class => AssocierCategoriesFormFactory::class,
            CategorieForm::class => CategorieFormFactory::class,
        ],
    ],
    'hydrators' => [
        'factories' => [
            AssocierCategoriesHydrator::class => AssocierCategoriesHydratorFactory::class,
            CategorieHydrator::class => CategorieHydratorFactory::class,
        ],
    ],

];