<?php
/**
 * Zend Framework (http://framework.zend.com/)
 *
 * @link      http://github.com/zendframework/ZendSkeletonApplication for the canonical source repository
 * @copyright Copyright (c] 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
 * @license   http://framework.zend.com/license/new-bsd New BSD License
 */

namespace Application;

use Application\Controller\ApplicationController;
use Application\Controller\CategorieController;
use Application\Controller\IndexController;
use Application\Controller\IndexControllerFactory;
use Doctrine\ORM\Mapping\Driver\XmlDriver;
use Doctrine\Persistence\Mapping\Driver\MappingDriverChain;
use UnicaenPrivilege\Guard\PrivilegeController;

return [
    'doctrine' => [
        'driver' => [
            'orm_default' => [
                'class' => MappingDriverChain::class,
                'drivers' => [
                    'Application\Entity\Db' => 'orm_default_xml_driver',
                ],
            ],
            'orm_default_xml_driver' => [
                'class' => XmlDriver::class,
                'cache' => 'apc',
                'paths' => [
                    __DIR__ . '/../src/Application/Entity/Db/Mapping',
                ],
            ],
        ],
        'cache' => [
            'apc' => [
                'namespace' => 'PREECOG__' . __NAMESPACE__,
            ],
        ],
    ],

    'bjyauthorize' => [
        'guards' => [
            PrivilegeController::class => [
                [
                    'controller' => IndexController::class,
                    'action' => [
                        'index',
                    ],
                    'roles' => 'guest'
                ],
                [
                    'controller' => IndexController::class,
                    'action' => [
                        'administration',
                    ],
                    'roles' => 'Administrateur·trice'
                ],
            ],
        ],
    ],

    'navigation' => [
        'default' => [
            'home' => [
                'pages' => [
                    'ressources' => [
                        'order' => 1000,
                        'label' => 'Ressources',
                        'title' => "ressources",
                        'route' => 'home',
                        'pages' => [
                            'application' => [
                                'label' => 'Applications',
                                'route' => 'application',
                                'resource' => PrivilegeController::getResourceId(ApplicationController::class, 'index'),
                                'order'    => 1000,
                                'icon' => 'fas fa-angle-right',
                            ],
                            'categorie' => [
                                'label' => 'Catégories',
                                'route' => 'categorie',
                                'resource' => PrivilegeController::getResourceId(CategorieController::class, 'index'),
                                'order'    => 1000,
                                'icon' => 'fas fa-angle-right',
                            ],
                        ],
                    ],
                ],
            ],
        ],
    ],

    'router'          => [
        'routes' => [
            'home'        => [
                'type'    => 'Literal',
                'options' => [
                    'route'    => '/',
                    'defaults' => [
                        /** @see IndexController::indexAction() */
                        'controller' => IndexController::class, // <-- change here
                        'action'     => 'index',
                    ],
                ],
            ],
            'administration'        => [
                'type'    => 'Literal',
                'options' => [
                    'route'    => '/administration',
                    'defaults' => [
                        /** @see IndexController::administrationAction() */
                        'controller' => IndexController::class,
                        'action'     => 'administration',
                    ],
                ],
            ],
        ],
    ],
    'controllers'     => [
        'factories' => [
            IndexController::class => IndexControllerFactory::class,
        ],
    ],
    'service_manager' => [
        'factories' => [

        ],
    ],
    'translator'      => [
        'locale'                    => 'fr_FR', // en_US
        'translation_file_patterns' => [
            [
                'type'     => 'gettext',
                'base_dir' => __DIR__ . '/../language',
                'pattern'  => '%s.mo',
            ],
        ],
    ],
    'view_manager'    => [
        'template_map'        => [
            'layout/layout' => __DIR__ . '/../view/layout/layout.phtml',
        ],
        'template_path_stack' => [
            __DIR__ . '/../view',
        ],
    ],



    'public_files' => [
        'head_scripts' => [
            '015_jquery' => 'unistrap-1.0.0/js/jquery-3.6.1.min.js',
            '040_bootstrap' => '',
        ],
        'inline_scripts' => [
            '020_app' => '',
            '030_util' => '/unicaen/app/js/util.js',
            '040_unicaen' => '/unicaen/app/js/unicaen.js',
            '050_jquery_form' => '',
            '070_bootstrap' => 'unistrap-1.0.0/lib/bootstrap-5.2.2/dist/js/bootstrap.bundle.min.js',
            '080_unistrap' => 'unistrap-1.0.0/js/unistrap.js',
            '110_' => 'vendor/DataTables-1.12.1/datatables.min.js',
            '120_bootstrap-select' => '/vendor/bootstrap-select-1.14.0-beta3/js/bootstrap-select.min.js',
            '120_bootstrap-select-fr' => '/vendor/bootstrap-select-1.14.0-beta3/js/i18n/defaults-fr_FR.js',
            '150_' => 'js/tinymce/js/tinymce/tinymce.js',
            '151_' => 'js/form_fiche.js',
        ],
        'stylesheets' => [
            '000_unistra' => 'css/font-local.css',
            '040_bootstrap' => 'unistrap-1.0.0/lib/bootstrap-5.2.2/dist/css/bootstrap.min.css',
            '041_ubuntu' => 'unistrap-1.0.0/css/font-ubuntu.css',
            '042_unistrap' => 'unistrap-1.0.0/css/unistrap.css',
            '060_unicaen' => '',
            '110_' => 'vendor/DataTables-1.12.1/datatables.min.css',
            '112_' => 'vendor/font-awesome-5.15.2/css/all.min.css',
            '120_' => '/vendor/bootstrap-select-1.14.0-beta3/css/bootstrap-select.min.css',
            '065_unicaen-icon' => '/unicaen/app/css/unicaen-icon.css',

            '976_environnement' => 'css/environnement.css',
            '977_app' => 'css/app.css',
            '998_default' => 'css/default.css',
            '999_local' => 'css/local.css',
        ],
    ],
];
