<?php

namespace Application\Controller;


use Application\Entity\Db\Application;
use Application\Form\Application\ApplicationFormAwareTrait;
use Application\Form\AssocierCategories\AssocierCategoriesFormAwareTrait;
use Application\Service\Application\ApplicationServiceAwareTrait;
use Laminas\Http\Response;
use Laminas\Mvc\Controller\AbstractActionController;
use Laminas\View\Model\ViewModel;

class ApplicationController extends AbstractActionController {
    use ApplicationServiceAwareTrait;
    use ApplicationFormAwareTrait;
    use AssocierCategoriesFormAwareTrait;

    public function indexAction(): ViewModel
    {
        $applications = $this->getApplicationService()->getApplications('ordre', 'ASC', true);

        return new ViewModel([
            'applications' => $applications,
        ]);
    }

    public function afficherAction(): ViewModel
    {
        $application = $this->getApplicationService()->getRequestedApplication($this);

        return new ViewModel([
            'application' => $application,
        ]);
    }


    public function ajouterAction(): ViewModel
    {
        $application = new Application();
        $form = $this->getApplicationForm();
        $form->setAttribute('action', $this->url()->fromRoute('application/ajouter', [], [], true));
        $form->bind($application);

        $request = $this->getRequest();
        if ($request->isPost()) {
            $data = $request->getPost();
            $form->setData($data);
            if ($form->isValid()) {
                $this->getApplicationService()->create($application);
                exit();
            }
        }

        $vm =  new ViewModel();
        $vm->setTemplate('default/default-form');
        $vm->setVariables([
            'title' => "Ajout d'une application",
            'form' => $form,
        ]);
        return $vm;
    }

    public function modifierAction(): ViewModel
    {
        $application = $this->getApplicationService()->getRequestedApplication($this);
        $form = $this->getApplicationForm();
        $form->setAttribute('action', $this->url()->fromRoute('application/modifier', ['application' => $application->getId()], [], true));
        $form->bind($application);

        $request = $this->getRequest();
        if ($request->isPost()) {
            $data = $request->getPost();
            $form->setData($data);
            if ($form->isValid()) {
                $this->getApplicationService()->update($application);
                exit();
            }
        }

        $vm =  new ViewModel();
        $vm->setTemplate('default/default-form');
        $vm->setVariables([
            'title' => "Modification d'une application",
            'form' => $form,
        ]);
        return $vm;
    }

    public function historiserAction(): Response
    {
        $application = $this->getApplicationService()->getRequestedApplication($this);
        $this->getApplicationService()->historise($application);

        $retour = $this->params()->fromQuery('retour');
        if ($retour) return $this->redirect()->toUrl($retour);
        return $this->redirect()->toRoute('application', [], [], true);

    }

    public function restaurerAction(): Response
    {
        $application = $this->getApplicationService()->getRequestedApplication($this);
        $this->getApplicationService()->restore($application);

        $retour = $this->params()->fromQuery('retour');
        if ($retour) return $this->redirect()->toUrl($retour);
        return $this->redirect()->toRoute('application', [], [], true);
    }

    public function supprimerAction()
    {
        $application = $this->getApplicationService()->getRequestedApplication($this);

        $request = $this->getRequest();
        if ($request->isPost()) {
            $data = $request->getPost();
            if ($data["reponse"] === "oui") $this->getApplicationService()->delete($application);
            exit();
        }

        $vm = new ViewModel();
        if ($application !== null) {
            $vm->setTemplate('default/confirmation');
            $vm->setVariables([
                'title' => "Suppression de l'application " . $application->getLibelle(),
                'text' => "La suppression est définitive êtes-vous sûr&middot;e de vouloir continuer ?",
                'action' => $this->url()->fromRoute('application/supprimer', ["application" => $application->getId()], [], true),
            ]);
        }
        return $vm;
    }

    public function associerCategoriesAction(): ViewModel
    {
        $application = $this->getApplicationService()->getRequestedApplication($this);
        $form = $this->getAssocierCategoriesForm();
        $form->setAttribute('action', $this->url()->fromRoute('application/associer-categories', ['application' => $application->getId()], [], true));
        $form->bind($application);

        $request = $this->getRequest();
        if ($request->isPost()) {
            $data = $request->getPost();
            if (!isset($data['categories'])) $data['categories'] = [];
            $form->setData($data);
            if ($form->isValid()) {
                $this->getApplicationService()->update($application);
                exit();
            }
        }

        $vm =  new ViewModel();
        $vm->setTemplate('default/default-form');
        $vm->setVariables([
            'title' => "Associer des catégories à une application",
            'form' => $form,
        ]);
        return $vm;
    }
}