<?php

namespace Application\Controller;

use Application\Form\Application\ApplicationForm;
use Application\Form\AssocierCategories\AssocierCategoriesForm;
use Application\Service\Application\ApplicationService;
use Interop\Container\ContainerInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\NotFoundExceptionInterface;

class ApplicationControllerFactory {

    /**
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container): ApplicationController
    {
        /**
         * @var ApplicationService $applicationService
         * @var ApplicationForm $applicationForm
         * @var AssocierCategoriesForm $associerCategorieForm
         */
        $applicationService = $container->get(ApplicationService::class);
        $applicationForm = $container->get('FormElementManager')->get(ApplicationForm::class);
        $associerCategorieForm = $container->get('FormElementManager')->get(AssocierCategoriesForm::class);


        $controller = new ApplicationController();
        $controller->setApplicationService($applicationService);
        $controller->setApplicationForm($applicationForm);
        $controller->setAssocierCategoriesForm($associerCategorieForm);
        return $controller;
    }
}