<?php

namespace Application\Controller;


use Application\Entity\Db\Categorie;
use Application\Form\AssocierApplications\AssocierApplicationsFormAwareTrait;
use Application\Form\Categorie\CategorieFormAwareTrait;
use Application\Service\Categorie\CategorieServiceAwareTrait;
use Laminas\Http\Response;
use Laminas\Mvc\Controller\AbstractActionController;
use Laminas\View\Model\ViewModel;

class CategorieController extends AbstractActionController {
    use CategorieServiceAwareTrait;
    use CategorieFormAwareTrait;
    use AssocierApplicationsFormAwareTrait;

    public function indexAction(): ViewModel
    {
        $categories = $this->getCategorieService()->getCategories();

        return new ViewModel([
            'categories' => $categories,
        ]);
    }

    public function afficherAction(): ViewModel
    {
        $categorie = $this->getCategorieService()->getRequestedCategorie($this);

        return new ViewModel([
            'categorie' => $categorie,
        ]);
    }

    public function ajouterAction(): ViewModel
    {
        $categorie = new Categorie();
        $form = $this->getCategorieForm();
        $form->setAttribute('action', $this->url()->fromRoute('categorie/ajouter', [], [], true));
        $form->bind($categorie);

        $request = $this->getRequest();
        if ($request->isPost()) {
            $data = $request->getPost();
            $form->setData($data);
            if ($form->isValid()) {
                $this->getCategorieService()->create($categorie);
                exit();
            }
        }

        $vm =  new ViewModel();
        $vm->setTemplate('default/default-form');
        $vm->setVariables([
            'title' => "Ajout d'une catégorie d'applications",
            'form' => $form,
        ]);
        return $vm;
    }

    public function modifierAction(): ViewModel
    {
        $categorie = $this->getCategorieService()->getRequestedCategorie($this);
        $form = $this->getCategorieForm();
        $form->setAttribute('action', $this->url()->fromRoute('categorie/modifier', ['categorie' => $categorie->getId()], [], true));
        $form->bind($categorie);

        $request = $this->getRequest();
        if ($request->isPost()) {
            $data = $request->getPost();
            $form->setData($data);
            if ($form->isValid()) {
                $this->getCategorieService()->update($categorie);
                exit();
            }
        }

        $vm =  new ViewModel();
        $vm->setTemplate('default/default-form');
        $vm->setVariables([
            'title' => "Modification d'une catégorie d'applications",
            'form' => $form,
        ]);
        return $vm;
    }

    public function historiserAction(): Response
    {
        $categorie = $this->getCategorieService()->getRequestedCategorie($this);
        $this->getCategorieService()->historise($categorie);

        $retour = $this->params()->fromQuery('retour');
        if ($retour) return $this->redirect()->toUrl($retour);
        return $this->redirect()->toRoute('application', [], [], true);


    }

    public function restaurerAction(): Response
    {
        $categorie = $this->getCategorieService()->getRequestedCategorie($this);
        $this->getCategorieService()->restore($categorie);

        $retour = $this->params()->fromQuery('retour');
        if ($retour) return $this->redirect()->toUrl($retour);
        return $this->redirect()->toRoute('application', [], [], true);
    }

    public function supprimerAction(): ViewModel
    {
        $categorie = $this->getCategorieService()->getRequestedCategorie($this);

        $request = $this->getRequest();
        if ($request->isPost()) {
            $data = $request->getPost();
            if ($data["reponse"] === "oui") $this->getCategorieService()->delete($categorie);
            exit();
        }

        $vm = new ViewModel();
        if ($categorie !== null) {
            $vm->setTemplate('default/confirmation');
            $vm->setVariables([
                'title' => "Suppression de la catégorie " . $categorie->getLibelle(),
                'text' => "La suppression est définitive êtes-vous sûr&middot;e de vouloir continuer ?",
                'action' => $this->url()->fromRoute('categorie/supprimer', ["categorie" => $categorie->getId()], [], true),
            ]);
        }
        return $vm;
    }

    public function associerApplicationsAction(): ViewModel
    {
        $categorie = $this->getCategorieService()->getRequestedCategorie($this);

        $form = $this->getAssocierApplicationsForm();
        $form->setAttribute('action', $this->url()->fromRoute('categorie/associer-applications', ['categorie' => $categorie->getId()], [], true));
        $form->bind($categorie);

        $request = $this->getRequest();
        if ($request->isPost()) {
            $data = $request->getPost();
            if (!isset($data['categories'])) $data['categories'] = [];
            $form->setData($data);
            if ($form->isValid()) {
                $this->getCategorieService()->update($categorie);
                exit();
            }
        }

        $vm =  new ViewModel();
        $vm->setTemplate('default/default-form');
        $vm->setVariables([
            'title' => "Associer des applications à une catégorie",
            'form' => $form,
        ]);
        return $vm;
    }
}