<?php

namespace Application\Service\Application;

use Application\Entity\Db\Application;
use Doctrine\ORM\NonUniqueResultException;
use Doctrine\ORM\QueryBuilder;
use DoctrineModule\Persistence\ProvidesObjectManager;
use Laminas\Mvc\Controller\AbstractActionController;
use UnicaenApp\Exception\RuntimeException;

class ApplicationService
{
    use ProvidesObjectManager;

    /** GESTION DES ENTITES *******************************************************************************************/

    public function create(Application $application): Application
    {
        $this->getObjectManager()->persist($application);
        $this->getObjectManager()->flush();
        return $application;
    }

    public function update(Application $application): Application
    {
        $this->getObjectManager()->flush();
        return $application;
    }

    public function historise(Application $application): Application
    {
        $application->historiser();
        $this->getObjectManager()->flush();
        return $application;
    }

    public function restore(Application $application): Application
    {
        $application->dehistoriser();
        $this->getObjectManager()->flush();
        return $application;
    }

    public function delete(Application $application): Application
    {
        $this->getObjectManager()->remove($application);
        $this->getObjectManager()->flush();
        return $application;
    }

    /** REQUETAGE *****************************************************************************************************/

    public function createQueryBuilder(string $alias = 'application'): QueryBuilder
    {
        $qb = $this->getObjectManager()->getRepository(Application::class)->createQueryBuilder($alias)
            ->addSelect('categorie')->leftJoin('application.categories', 'categorie');

        return $qb;
    }

    /** @return Application[] */
    public function getApplications(string $champ = 'libelle', string $ordre = 'ASC', bool $withHisto = false): array
    {
        $qb = $this->createQueryBuilder()
            ->orderBy('application.' . $champ, $ordre);
        if ($withHisto) $qb = $qb->andWhere('application.histoDestruction IS NULL');

        $result = $qb->getQuery()->getResult();
        return $result;
    }

    /** @return string[] */
    public function getApplicationsAsOptions(string $champ = 'libelle', string $ordre = 'ASC', bool $withHisto = false): array
    {
        $applications = $this->getApplications($champ, $ordre,$withHisto);
        $array = [];
        foreach ($applications as $application) {
            $array[$application->getId()] = $application->getLibelle();
        }
        return $array;
    }

    public function getApplication(?int $id): ?Application
    {
        $qb = $this->createQueryBuilder()
            ->andWhere('application.id = :id')
            ->setParameter('id', $id);

        try {
            $result = $qb->getQuery()->getOneOrNullResult();
        } catch (NonUniqueResultException $e) {
            throw new RuntimeException("Plusieurs [".self::class."] partagent le même id [".$id."]",0,$e);
        }
        return $result;
    }

    public function getRequestedApplication(AbstractActionController $controller, $paramName = 'application'): ?Application
    {
        $id = $controller->params()->fromRoute($paramName);
        $result = $this->getApplication($id);
        return $result;
    }

    /** @return Application[] */
    public function getApplicationsSansCategories(string $champ = 'ordre', string $ordre = 'ASC', bool $withHisto = false): array
    {
        $qb = $this->createQueryBuilder()
            ->andWhere('categorie.id IS NULL')
            ->orderBy('application.' . $champ, $ordre)
        ;
        if (!$withHisto) $qb = $qb->andWhere('application.histoDestruction IS NULL');
        $result = $qb->getQuery()->getResult();
        return $result;
    }

}