<?php

namespace Application\Service\Categorie;


use Application\Entity\Db\Categorie;
use Doctrine\ORM\NonUniqueResultException;
use Doctrine\ORM\QueryBuilder;
use DoctrineModule\Persistence\ProvidesObjectManager;
use Laminas\Mvc\Controller\AbstractActionController;
use RuntimeException;

class CategorieService
{
    use ProvidesObjectManager;

    /** GESTION DES ENTITES *******************************************************************************************/

    public function create(Categorie $categorie): Categorie
    {
        $this->getObjectManager()->persist($categorie);
        $this->getObjectManager()->flush();
        return $categorie;
    }

    public function update(Categorie $categorie): Categorie
    {
        $this->getObjectManager()->flush();
        return $categorie;
    }

    public function historise(Categorie $categorie): Categorie
    {
        $categorie->historiser();
        $this->getObjectManager()->flush();
        return $categorie;
    }

    public function restore(Categorie $categorie): Categorie
    {
        $categorie->dehistoriser();
        $this->getObjectManager()->flush();
        return $categorie;
    }

    public function delete(Categorie $categorie): Categorie
    {
        $this->getObjectManager()->remove($categorie);
        $this->getObjectManager()->flush();
        return $categorie;
    }

    /** REQUETAGE *****************************************************************************************************/

    public function createQueryBuilder(string $alias = 'categorie'): QueryBuilder
    {
        $qb = $this->getObjectManager()->getRepository(Categorie::class)->createQueryBuilder($alias)
            ->addSelect('application')->leftJoin('categorie.applications', 'application')
        ;
        return $qb;
    }

    /** @return Categorie[] */
    public function getCategories(string $champ = 'libelle', string $ordre = 'ASC', bool $withHisto = false): array
    {
        $qb = $this->createQueryBuilder()
            ->orderBy('categorie.' . $champ, $ordre);
        if (!$withHisto) $qb = $qb->andWhere('categorie.histoDestruction IS NULL');

        $result = $qb->getQuery()->getResult();
        return $result;
    }

    /** @return string[] */
    public function getCategoriesAsOptions(string $champ = 'libelle', string $ordre = 'ASC', bool $withHisto = false): array
    {
        $categories = $this->getCategories($champ, $ordre, $withHisto);
        $array = [];
        foreach ($categories as $categorie) {
            $array[$categorie->getId()] = $categorie->getLibelle();
        }
        return $array;
    }

    public function getCategorie(?int $id): ?Categorie
    {
        $qb = $this->createQueryBuilder()
            ->andWhere('categorie.id = :id')
            ->setParameter('id', $id)
        ;

        try {
            $result = $qb->getQuery()->getOneOrNullResult();
        } catch (NonUniqueResultException $e) {
            throw new RuntimeException("Plusieurs [".self::class."] partagent le même id [".$id."]",0,$e);
        }
        return $result;
    }

    public function getRequestedCategorie(AbstractActionController $controller, string $paramName = 'categorie'): ?Categorie
    {
        $id = $controller->params()->fromRoute($paramName);
        $result = $this->getCategorie($id);
        return $result;
    }

}